# ======================================================================
# Funo para avaliao por N-fold Cross Validation
#
cross.validation <- function(form,all.data,cv.folds=10) {

  cat('\nStarting a ',cv.folds,' -fold CV process.\nFold ::')
  n <- nrow(all.data)
  n.each.part <- as.integer(n/cv.folds)
  
  permutation <- sample(n,n)
  perm.data <- all.data[permutation,]

  for(i in seq(cv.folds)) {
    cat(' ',i)
      
    out.fold <- ((i-1)*n.each.part+1):(i*n.each.part)
    
    it.res <- cv.iteration(form,perm.data[-out.fold,],perm.data[out.fold,])
    
    if (exists('results'))
      results[i,] <- it.res
    else 
      results <- data.frame(it.res)
  }
  cat('\n')
  
  results[cv.folds+1,] <- apply(results,2,mean)
  results[cv.folds+2,] <- apply(results,2,sd)
  row.names(results) <- c(seq(cv.folds),'avg','std')
  results
}


cv.iteration <- function(form,train,test) {
  require(rpart)

  # Extrair o nome da variavel objectivo
  mt <- terms(form, data = train)
  all.vars <- as.character(attr(mt, "variables"))[-1]
  target <- all.vars[attr(mt,"response")]

  t <- system.time(model <- rpart(form,train))[1]
  t <- t + system.time(preds <- predict(model,test))[1]
  
  list(mae=mean(abs(preds-test[,target])),tempo=t)
}


mdl.real <- function(x,prec=0.0001) {
  n <- x/prec
  ceiling(1+mdl.real.aux(n)+log2(2.865064))
}

mdl.real.aux <- function(n) {
  x <- log2(n)
  ifelse(x <=0 , 0, x + mdl.real.aux(x))
}
